@@ -13,12 +13,10 @@ These descriptions are not always clear and/or precise and when assumptions have
 been made they are explained in the strategy docstrings.
 """
 
-import random
-from typing import Dict, List, Tuple, Optional
+from typing import Dict, List, Optional, Tuple
 
 from axelrod.action import Action
 from axelrod.player import Player
-from axelrod.random_ import random_choice
 from axelrod.strategy_transformers import FinalTransformer
 from scipy.stats import chisquare
 
@@ -47,7 +45,6 @@ class FirstByDavis(Player):
     classifier = {
         "memory_depth": float("inf"),  # Long memory
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -236,7 +233,6 @@ class FirstByDowning(Player):
     classifier = {
         "memory_depth": float("inf"),
         "stochastic": False,
-        "makes_use_of": {"game"},
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -311,7 +307,6 @@ class FirstByFeld(Player):
     classifier = {
         "memory_depth": 200,  # Varies actually, eventually becomes depth 1
         "stochastic": True,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -355,7 +350,7 @@ class FirstByFeld(Player):
         if opponent.history[-1] == D:
             return D
         p = self._cooperation_probability()
-        return random_choice(p)
+        return self._random.random_choice(p)
 
 
 class FirstByGraaskamp(Player):
@@ -403,7 +398,6 @@ class FirstByGraaskamp(Player):
     classifier = {
         "memory_depth": float("inf"),
         "stochastic": True,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -450,11 +444,11 @@ class FirstByGraaskamp(Player):
             return C
 
         if self.next_random_defection_turn is None:
-            self.next_random_defection_turn = random.randint(5, 15) + len(self.history)
+            self.next_random_defection_turn = self._random.randint(5, 15) + len(self.history)
 
         if len(self.history) == self.next_random_defection_turn:
             # resample the next defection turn
-            self.next_random_defection_turn = random.randint(5, 15) + len(self.history)
+            self.next_random_defection_turn = self._random.randint(5, 15) + len(self.history)
             return D
         return C
 
@@ -479,16 +473,16 @@ class FirstByGrofman(Player):
     classifier = {
         "memory_depth": 1,
         "stochastic": True,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
         "manipulates_state": False,
     }
+
     def strategy(self, opponent: Player) -> Action:
         if len(self.history) == 0 or self.history[-1] == opponent.history[-1]:
             return C
-        return random_choice(2 / 7)
+        return self._random.random_choice(2 / 7)
 
 
 class FirstByJoss(MemoryOnePlayer):
@@ -519,7 +513,6 @@ class FirstByJoss(MemoryOnePlayer):
             or (D, C), i.e. the opponent cooperated.
         """
         four_vector = (p, 0, p, 0)
-        self.p = p
         super().__init__(four_vector)
 
 
@@ -580,7 +573,6 @@ class FirstByNydegger(Player):
     classifier = {
         "memory_depth": 3,
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -667,7 +659,6 @@ class FirstByShubik(Player):
     classifier = {
         "memory_depth": float("inf"),
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -738,7 +729,6 @@ class FirstByTullock(Player):
     classifier = {
         "memory_depth": float("inf"),
         "stochastic": True,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -746,15 +736,8 @@ class FirstByTullock(Player):
     }
 
     def __init__(self) -> None:
-        """
-        Parameters
-        ----------
-        rounds_to_cooperate: int
-           The number of rounds to cooperate initially
-        """
         super().__init__()
         self._rounds_to_cooperate = 11
-        self.memory_depth = self._rounds_to_cooperate
 
     def strategy(self, opponent: Player) -> Action:
         if len(self.history) < self._rounds_to_cooperate:
@@ -763,7 +746,7 @@ class FirstByTullock(Player):
         cooperate_count = opponent.history[-rounds:].count(C)
         prop_cooperate = cooperate_count / rounds
         prob_cooperate = max(0, prop_cooperate - 0.10)
-        return random_choice(prob_cooperate)
+        return self._random.random_choice(prob_cooperate)
 
 
 class FirstByAnonymous(Player):
@@ -794,17 +777,15 @@ class FirstByAnonymous(Player):
     classifier = {
         "memory_depth": 0,
         "stochastic": True,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
         "manipulates_state": False,
     }
 
-    @staticmethod
-    def strategy(opponent: Player) -> Action:
-        r = random.uniform(3, 7) / 10
-        return random_choice(r)
+    def strategy(self, opponent: Player) -> Action:
+        r = self._random.uniform(3, 7) / 10
+        return self._random.random_choice(r)
 
 
 @FinalTransformer((D, D), name_prefix=None)
@@ -839,7 +820,6 @@ class FirstBySteinAndRapoport(Player):
     classifier = {
         "memory_depth": float("inf"),
         "stochastic": False,
-        "makes_use_of": {"length"},
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -927,7 +907,6 @@ class FirstByTidemanAndChieruzzi(Player):
     classifier = {
         "memory_depth": float("inf"),
         "stochastic": False,
-        "makes_use_of": {"game", "length"},
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
