@@ -1,13 +1,10 @@
-import random
-
 import numpy as np
-from numpy.random import choice
-
 from axelrod.action import Action
 from axelrod.classifier import Classifiers
 from axelrod.player import Player
 from axelrod.strategies import TitForTat
 from axelrod.strategy_transformers import NiceTransformer
+
 from ._strategies import all_strategies
 from .hunter import (
     AlternatorHunter,
@@ -20,7 +17,9 @@ from .hunter import (
 )
 
 # Needs to be computed manually to prevent circular dependency
-ordinary_strategies = [s for s in all_strategies if Classifiers.obey_axelrod(s())]
+ordinary_strategies = [
+    s for s in all_strategies if Classifiers.obey_axelrod(s())
+]
 
 C, D = Action.C, Action.D
 
@@ -38,7 +37,6 @@ class MetaPlayer(Player):
     classifier = {
         "memory_depth": float("inf"),  # Long memory
         "stochastic": True,
-        "makes_use_of": set(),
         "long_run_time": True,
         "inspects_source": False,
         "manipulates_source": False,
@@ -52,16 +50,16 @@ class MetaPlayer(Player):
         if team:
             self.team = team
         else:
-            # Needs to be computed manually to prevent circular dependency
             self.team = ordinary_strategies
         # Make sure we don't use any meta players to avoid infinite recursion.
         self.team = [t for t in self.team if not issubclass(t, MetaPlayer)]
         # Initiate all the players in our team.
         self.team = [t() for t in self.team]
-
+        self._last_results = None
         super().__init__()
 
-        # This player inherits the classifiers of its team.
+    def _post_init(self):
+        # The player's classification characteristics are derived from the team.
         # Note that memory_depth is not simply the max memory_depth of the team.
         for key in [
             "stochastic",
@@ -71,10 +69,17 @@ class MetaPlayer(Player):
         ]:
             self.classifier[key] = any(map(Classifiers[key], self.team))
 
+        self.classifier["makes_use_of"] = set()
         for t in self.team:
-            self.classifier["makes_use_of"].update(Classifiers["makes_use_of"](t))
+            new_uses = Classifiers["makes_use_of"](t)
+            if new_uses:
+                self.classifier["makes_use_of"].update(new_uses)
 
-        self._last_results = None
+    def set_seed(self, seed=None):
+        super().set_seed(seed=seed)
+        # Seed the team as well
+        for t in self.team:
+            t.set_seed(self._random.random_seed_int())
 
     def receive_match_attributes(self):
         for t in self.team:
@@ -88,8 +93,22 @@ class MetaPlayer(Player):
 
     def update_histories(self, coplay):
         # Update team histories.
-        for player, play in zip(self.team, self._last_results):
-            player.history.append(play, coplay)
+        try:
+            for player, play in zip(self.team, self._last_results):
+                player.update_history(play, coplay)
+        except TypeError:
+            # If the Meta class is decorated by the Joss-Ann transformer,
+            # such that the decorated class is now deterministic, the underlying
+            # strategy isn't called. In that case, updating the history of all the
+            # team members doesn't matter.
+            # As a sanity check, look for at least one reclassifier, otherwise
+            # this try-except clause could hide a bug.
+            if len(self._reclassifiers) == 0:
+                raise TypeError("MetaClass update_histories issue, expected a reclassifier.")
+            # Otherwise just update with C always, so at least the histories have the
+            # expected length.
+            for player in self.team:
+                player.update_history(C, coplay)
 
     def update_history(self, play, coplay):
         super().update_history(play, coplay)
@@ -179,7 +198,7 @@ class MetaWinner(MetaPlayer):
         self._update_scores(coplay)
 
     def meta_strategy(self, results, opponent):
-        # Choice an action based on the collection of scores
+        # Choose an action based on the collection of scores
         bestscore = max(self.scores)
         beststrategies = [
             i for (i, score) in enumerate(self.scores) if score == bestscore
@@ -195,7 +214,7 @@ NiceMetaWinner = NiceTransformer()(MetaWinner)
 class MetaWinnerEnsemble(MetaWinner):
     """A variant of MetaWinner that chooses one of the top scoring strategies
     at random against each opponent. Note this strategy is always stochastic
-    regardless of the team.
+    regardless of the team, if team larger than 1, and the players are distinct.
 
     Names:
 
@@ -204,13 +223,32 @@ class MetaWinnerEnsemble(MetaWinner):
 
     name = "Meta Winner Ensemble"
 
+    def _post_init(self):
+        super()._post_init()
+        team = list(t.__class__ for t in self.team)
+        if len(team) > 1:
+            self.classifier["stochastic"] = True
+            self.singular = False
+        else:
+            self.singular = True
+        # If the team has repeated identical members, then it reduces to a singular team
+        # and it may not actually be stochastic.
+        if team and len(set(team)) == 1:
+            self.classifier["stochastic"] = Classifiers["stochastic"](self.team[0])
+            self.singular = True
+
     def meta_strategy(self, results, opponent):
+        # If the team consists of identical players, just take the first result.
+        # This prevents an unnecessary call to _random below.
+        if self.singular:
+            return results[0]
         # Sort by score
         scores = [(score, i) for (i, score) in enumerate(self.scores)]
         # Choose one of the best scorers at random
         scores.sort(reverse=True)
         prop = max(1, int(len(scores) * 0.08))
-        index = choice([i for (s, i) in scores[:prop]])
+        best_scorers = [i for (s, i) in scores[:prop]]
+        index = self._random.choice(best_scorers)
         return results[index]
 
 
@@ -229,7 +267,6 @@ class MetaHunter(MetaPlayer):
     classifier = {
         "memory_depth": float("inf"),  # Long memory
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -283,7 +320,6 @@ class MetaHunterAggressive(MetaPlayer):
     classifier = {
         "memory_depth": float("inf"),  # Long memory
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -332,7 +368,11 @@ class MetaMajorityMemoryOne(MetaMajority):
     name = "Meta Majority Memory One"
 
     def __init__(self):
-        team = [s for s in ordinary_strategies if Classifiers["memory_depth"](s()) <= 1]
+        team = [
+            s
+            for s in ordinary_strategies
+            if Classifiers["memory_depth"](s()) <= 1
+        ]
         super().__init__(team=team)
         self.classifier["long_run_time"] = False
 
@@ -386,7 +426,11 @@ class MetaWinnerMemoryOne(MetaWinner):
     name = "Meta Winner Memory One"
 
     def __init__(self):
-        team = [s for s in ordinary_strategies if Classifiers["memory_depth"](s()) <= 1]
+        team = [
+            s
+            for s in ordinary_strategies
+            if Classifiers["memory_depth"](s()) <= 1
+        ]
         super().__init__(team=team)
         self.classifier["long_run_time"] = False
 
@@ -440,7 +484,9 @@ class MetaWinnerDeterministic(MetaWinner):
     name = "Meta Winner Deterministic"
 
     def __init__(self):
-        team = [s for s in ordinary_strategies if not Classifiers["stochastic"](s())]
+        team = [
+            s for s in ordinary_strategies if not Classifiers["stochastic"](s())
+        ]
         super().__init__(team=team)
         self.classifier["stochastic"] = False
 
@@ -456,7 +502,9 @@ class MetaWinnerStochastic(MetaWinner):
     name = "Meta Winner Stochastic"
 
     def __init__(self):
-        team = [s for s in ordinary_strategies if Classifiers["stochastic"](s())]
+        team = [
+            s for s in ordinary_strategies if Classifiers["stochastic"](s())
+        ]
         super().__init__(team=team)
 
 
@@ -485,7 +533,6 @@ class MetaMixer(MetaPlayer):
     classifier = {
         "memory_depth": float("inf"),  # Long memory
         "stochastic": True,
-        "makes_use_of": set(),
         "long_run_time": True,
         "inspects_source": False,
         "manipulates_source": False,
@@ -493,12 +540,46 @@ class MetaMixer(MetaPlayer):
     }
 
     def __init__(self, team=None, distribution=None):
-        self.distribution = distribution
         super().__init__(team=team)
+        # Check that distribution is not all zeros, which will make numpy unhappy.
+        if distribution and all(x == 0 for x in distribution):
+            distribution = None
+        self.distribution = distribution
+
+    def _post_init(self):
+        distribution = self.distribution
+        if distribution and len(set(distribution)) > 1:
+            self.classifier["stochastic"] = True
+        if len(self.team) == 1:
+            self.classifier["stochastic"] = Classifiers["stochastic"](self.team[0])
+            # Overwrite strategy to avoid use of _random. This will ignore self.meta_strategy.
+            self.index = 0
+            self.strategy = self.index_strategy
+            return
+        # Check if the distribution has only one non-zero value. If so, the strategy may be
+        # deterministic, and we can avoid _random.
+        if distribution:
+            total = sum(distribution)
+            distribution = np.array(distribution) / total
+            if 1 in distribution:
+                self.index = list(distribution).index(1)
+                # It's potentially deterministic.
+                self.classifier["stochastic"] = Classifiers["stochastic"](self.team[self.index])
+                # Overwrite strategy to avoid use of _random. This will ignore self.meta_strategy.
+                self.strategy = self.index_strategy
+
+    def index_strategy(self, opponent):
+        """When the team effectively has a single player, only use that strategy."""
+        results = [C] * len(self.team)
+        player = self.team[self.index]
+        action = player.strategy(opponent)
+        results[self.index] = action
+        self._last_results = results
+        return action
 
     def meta_strategy(self, results, opponent):
-        """Using the numpy.random choice function to sample with weights"""
-        return choice(results, p=self.distribution)
+        """Using the _random.choice function to sample with weights."""
+        return self._random.choice(results, p=self.distribution)
 
 
 class NMWEDeterministic(NiceMetaWinnerEnsemble):
@@ -512,7 +593,9 @@ class NMWEDeterministic(NiceMetaWinnerEnsemble):
     name = "NMWE Deterministic"
 
     def __init__(self):
-        team = [s for s in ordinary_strategies if not Classifiers["stochastic"](s())]
+        team = [
+            s for s in ordinary_strategies if not Classifiers["stochastic"](s())
+        ]
         super().__init__(team=team)
         self.classifier["stochastic"] = True
 
@@ -528,7 +611,9 @@ class NMWEStochastic(NiceMetaWinnerEnsemble):
     name = "NMWE Stochastic"
 
     def __init__(self):
-        team = [s for s in ordinary_strategies if Classifiers["stochastic"](s())]
+        team = [
+            s for s in ordinary_strategies if Classifiers["stochastic"](s())
+        ]
         super().__init__(team=team)
 
 
@@ -581,7 +666,11 @@ class NMWEMemoryOne(NiceMetaWinnerEnsemble):
     name = "NMWE Memory One"
 
     def __init__(self):
-        team = [s for s in ordinary_strategies if Classifiers["memory_depth"](s()) <= 1]
+        team = [
+            s
+            for s in ordinary_strategies
+            if Classifiers["memory_depth"](s()) <= 1
+        ]
         super().__init__(team=team)
         self.classifier["long_run_time"] = False
 
@@ -610,7 +699,6 @@ class MemoryDecay(MetaPlayer):
         "memory_depth": float("inf"),
         "long_run_time": False,
         "stochastic": True,
-        "makes_use_of": set(),
         "inspects_source": False,
         "manipulates_source": False,
         "manipulates_state": False,
@@ -627,10 +715,6 @@ class MemoryDecay(MetaPlayer):
         start_strategy_duration: int = 15,
     ):
         super().__init__(team=[start_strategy])
-        # This strategy is stochastic even if none of the team is.  The
-        # MetaPlayer initializer will set stochastic to be False in that case.
-        self.classifier["stochastic"] = True
-
         self.p_memory_delete = p_memory_delete
         self.p_memory_alter = p_memory_alter
         self.loss_value = loss_value
@@ -639,6 +723,12 @@ class MemoryDecay(MetaPlayer):
         self.start_strategy_duration = start_strategy_duration
         self.gloss_values = None
 
+    def _post_init(self):
+        super()._post_init()
+        # This strategy is stochastic even if none of the team is.  The
+        # MetaPlayer initializer will set stochastic to be False in that case.
+        self.classifier["stochastic"] = True
+
     def __repr__(self):
         return Player.__repr__(self)
 
@@ -654,14 +744,14 @@ class MemoryDecay(MetaPlayer):
         """
         Alters memory entry, i.e. puts C if there's a D and vice versa.
         """
-        alter = choice(range(0, len(self.memory)))
+        alter = self._random.choice(range(0, len(self.memory)))
         self.memory[alter] = self.memory[alter].flip()
 
     def memory_delete(self):
         """
         Deletes memory entry.
         """
-        self.memory.pop(choice(range(0, len(self.memory))))
+        self.memory.pop(self._random.choice(range(0, len(self.memory))))
 
     def meta_strategy(self, results, opponent):
         try:
@@ -671,9 +761,9 @@ class MemoryDecay(MetaPlayer):
         if len(self.history) < self.start_strategy_duration:
             return results[0]
         else:
-            if random.random() <= self.p_memory_alter:
+            if self._random.random() <= self.p_memory_alter:
                 self.memory_alter()
-            if random.random() <= self.p_memory_delete:
+            if self._random.random() <= self.p_memory_delete:
                 self.memory_delete()
             self.gain_loss_translate()
             if sum(self.gloss_values) < 0:
