@@ -1,6 +1,5 @@
 from axelrod.action import Action, actions_to_str
 from axelrod.player import Player
-from axelrod.random_ import random_choice
 from axelrod.strategy_transformers import FinalTransformer, TrackHistoryTransformer
 
 C, D = Action.C, Action.D
@@ -29,7 +28,6 @@ class TitForTat(Player):
     classifier = {
         "memory_depth": 1,  # Four-Vector = (1.,0.,1.,0.)
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -65,7 +63,6 @@ class TitFor2Tats(Player):
     classifier = {
         "memory_depth": 2,  # Long memory, memory-2
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -90,7 +87,6 @@ class TwoTitsForTat(Player):
     classifier = {
         "memory_depth": 2,  # Long memory, memory-2
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -119,22 +115,20 @@ class DynamicTwoTitsForTat(Player):
     classifier = {
         "memory_depth": float("inf"),
         "stochastic": True,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
         "manipulates_state": False,
     }
 
-    @staticmethod
-    def strategy(opponent):
+    def strategy(self, opponent):
         # First move
         if not opponent.history:
             # Make sure we cooperate first turn
             return C
         if D in opponent.history[-2:]:
             # Probability of cooperating regardless
-            return random_choice(opponent.cooperations / len(opponent.history))
+            return self._random.random_choice(opponent.cooperations / len(opponent.history))
         else:
             return C
 
@@ -156,7 +150,6 @@ class Bully(Player):
     classifier = {
         "memory_depth": 1,  # Four-Vector = (0, 1, 0, 1)
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -180,7 +173,6 @@ class SneakyTitForTat(Player):
     classifier = {
         "memory_depth": float("inf"),  # Long memory
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -210,7 +202,6 @@ class SuspiciousTitForTat(Player):
     classifier = {
         "memory_depth": 1,  # Four-Vector = (1.,0.,1.,0.)
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -236,7 +227,6 @@ class AntiTitForTat(Player):
     classifier = {
         "memory_depth": 1,  # Four-Vector = (1.,0.,1.,0.)
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -260,7 +250,6 @@ class HardTitForTat(Player):
     classifier = {
         "memory_depth": 3,  # memory-three
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -292,7 +281,6 @@ class HardTitFor2Tats(Player):
     classifier = {
         "memory_depth": 3,  # memory-three
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -327,7 +315,6 @@ class OmegaTFT(Player):
     classifier = {
         "memory_depth": float("inf"),
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -408,7 +395,6 @@ class OriginalGradual(Player):
     classifier = {
         "memory_depth": float("inf"),
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -476,7 +462,6 @@ class Gradual(Player):
     classifier = {
         "memory_depth": float("inf"),
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -526,7 +511,6 @@ class ContriteTitForTat(Player):
     classifier = {
         "memory_depth": 3,
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -593,7 +577,6 @@ class AdaptiveTitForTat(Player):
     classifier = {
         "memory_depth": float("inf"),
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -637,7 +620,6 @@ class SpitefulTitForTat(Player):
     classifier = {
         "memory_depth": float("inf"),
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -679,7 +661,6 @@ class SlowTitForTwoTats2(Player):
     classifier = {
         "memory_depth": 2,
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -714,7 +695,6 @@ class Alexei(Player):
     classifier = {
         "memory_depth": float("inf"),
         "stochastic": False,
-        "makes_use_of": {"length"},
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -745,7 +725,6 @@ class EugineNier(Player):
     classifier = {
         "memory_depth": float("inf"),
         "stochastic": False,
-        "makes_use_of": {"length"},
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -782,7 +761,6 @@ class NTitsForMTats(Player):
     classifier = {
         "memory_depth": float("inf"),
         "stochastic": False,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -840,7 +818,6 @@ class Michaelos(Player):
     classifier = {
         "memory_depth": float("inf"),
         "stochastic": True,
-        "makes_use_of": {"length"},
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -857,7 +834,7 @@ class Michaelos(Player):
         if self.is_defector:
             return D
         if self.history[-1] == D and opponent.history[-1] == C:
-            decision = random_choice()
+            decision = self._random.random_choice()
             if decision == C:
                 return C
             else:
@@ -884,7 +861,6 @@ class RandomTitForTat(Player):
     classifier = {
         "memory_depth": 1,
         "stochastic": True,
-        "makes_use_of": set(),
         "long_run_time": False,
         "inspects_source": False,
         "manipulates_source": False,
@@ -911,7 +887,10 @@ class RandomTitForTat(Player):
 
         if self.act_random:
             self.act_random = False
-            return random_choice(self.p)
+            try:
+                return self._random.random_choice(self.p)
+            except AttributeError:
+                return D if self.p == 0 else C
 
         self.act_random = True
         return opponent.history[-1]
